@@ -10,10 +10,19 @@ from ..pattern import Pattern
 
 # External imports
 import numpy as np
-from shapely.geometry import Point, MultiPoint, LineString, MultiLineString
+from shapely.geometry import LineString, MultiLineString
 
 
 class Flank(FiringBase):
+    """Flank fire sets ignition paths in the direction of the wind.
+
+    Parameters
+    ----------
+    burn_unit : BurnUnit
+        Area bounding the ignition paths
+    ignition_crew : IgnitionCrew
+        Ignition crew assigned to the burn
+    """
 
     def __init__(self, burn_unit: BurnUnit, ignition_crew: IgnitionCrew):
         """Constructor
@@ -26,16 +35,29 @@ class Flank(FiringBase):
         # Initialize the base class
         super().__init__(burn_unit, ignition_crew)
 
-    def generate_pattern(self, depth=None, heat_depth=None) -> Pattern:
+    def generate_pattern(self, spacing: float = 0, depth: float = 0, heat_depth: float = 0, side: str = 'right', heat_delay: float = 0) -> Pattern:
         """Generate a flank fire ignition pattern
 
-        Returns:
-            Pattern: Spatiotemporal ignition pattern
-            depth (float): Depth in meters between igniters. If None, depth is computed by equally spacing igniters. Defaults to None.
-            heat_depth (float): Depth in meters between igniter heats. This argument is ignored if depth is None. Defaults to None.
+        Parameters
+        ----------
+        spacing : float
+            Staggering distance in meters between igniters within a heat
+        depth : float, optional
+            Depth in meters between igniters. Defaults to None.
+        heat_depth : float, optional
+            Depth in meters between heats. Defaults to None.
+        heat_delay : float, optional
+            Delay in seconds between heats. Defaults to 0.
+        side : str, optional
+            Side of the firing direction vector to start the ignition. Defaults to 'right'. Options are 'left' or 'right'.
+
+        Returns
+        -------
+        Pattern
+            Spatiotemporal ignition pattern
         """
 
-        return self._generate_pattern(depth=depth, heat_depth=heat_depth, return_trip=True)
+        return self._generate_pattern(spacing=spacing, depth=depth, heat_depth=heat_depth, side=side, return_trip=True, heat_delay=heat_delay)
 
     def _init_paths(self, paths: dict, **kwargs) -> dict:
         """Initialize spatial part of the ignition paths.
@@ -54,9 +76,10 @@ class Flank(FiringBase):
         # `generate_pattern()` method of this class)
         depth = kwargs['depth']
         heat_depth = kwargs['heat_depth']
+        side = kwargs['side']
 
         # Extract the bounding extent of the firing area
-        bbox = self._burn_unit.get_bounds()
+        bbox = self._burn_unit.bounds
         x_min, y_min = bbox[:, 0].min(), bbox[:, 1].min()
         x_max, y_max = bbox[:, 0].max(), bbox[:, 1].max()
 
@@ -84,6 +107,10 @@ class Flank(FiringBase):
                     cur_y = y_range[i] + depth
                 i += 1
 
+        # Flip the y_range if we're on the right side
+        if side == 'left':
+            y_range = y_range[::-1]
+
         # Initialize loop control parameters
         cur_heat = 0
         cur_igniter = 0
@@ -95,20 +122,19 @@ class Flank(FiringBase):
             line = LineString(((x_min, y), (x_max, y)))
             line = line.intersection(self._burn_unit.polygon)
 
+            # Get lines or multipart lines in the same list structure for looping below
+            if isinstance(line, LineString):
+                line_list = [line]
+            elif isinstance(line, MultiLineString):
+                line_list = list(line.geoms)
             # Edge case: In rare cases, the line along the top of the envelope becomes a point following
-            # the intersection (pretty sure this is a numerica precision issue). In this case, we need to
+            # the intersection (pretty sure this is a numerical precision issue). In this case, we need to
             # just skip this path.
-            if isinstance(line, (Point, MultiPoint)):
+            else:
                 continue
 
-            # Get lines or multipart lines in the same structure for looping below
-            if isinstance(line, LineString):
-                line = [line]
-            elif isinstance(line, MultiLineString):
-                line = list(line.geoms)
-
             # Assign the path to a heat, igniter and leg
-            for j, part in enumerate(line):
+            for j, part in enumerate(line_list):
                 paths['heat'].append(cur_heat)
                 paths['igniter'].append(cur_igniter)
                 paths['leg'].append(j)
