@@ -14,32 +14,45 @@ import numpy as np
 
 
 class Strip(FiringBase):
+    """Strip firing produces ignition paths perpendicular to the wind direction. Igniters are staggered with their heats
+    and each heat alternates on which side of the unit they start.
+
+    Parameters
+    ----------
+    burn_unit : BurnUnit
+        Area bounding the ignition paths
+    ignition_crew : IgnitionCrew
+        Ignition crew assigned to the burn
+    """
 
     def __init__(self, burn_unit: BurnUnit, ignition_crew: IgnitionCrew):
-        """Constructor
-
-        Args:
-            burn_unit (BurnUnit): Area bounding the ignition paths
-            ignition_crew (IgnitionCrew): Ignition crew assigned to the burn
-        """
 
         # Initialize the base class
         super().__init__(burn_unit, ignition_crew)
 
-    def generate_pattern(self, spacing: float, depth: float, heat_depth: float = None) -> Pattern:
-        """Generate a strip head fire ignition pattern.
+    def generate_pattern(self, spacing: float = 0, depth: float = 0, heat_depth: float = 0, side: str = 'right', heat_delay: float = 0) -> Pattern:
+        """Generate a flank fire ignition pattern
 
-        Args:
-            spacing (float): Staggering distance in meters between igniters within a heat
-            depth (float): Horizontal distance in meters between igniters and heats
-            heat_depth (float): Depth in meters between igniter heats. If None,
-                heat_depth is equal to igniter depth. Defaults to None.
+        Parameters
+        ----------
+        spacing : float, optional
+            Staggering distance in meters between igniters within a heat
+        depth : float, optional
+            Horizontal distance in meters between igniters and heats
+        heat_depth : float, optional
+            Depth in meters between igniter heats. If None, heat_depth is equal to igniter depth. Defaults to None.
+        heat_delay : float, optional
+            Delay in seconds between igniter heats. Defaults to 0.
+        side : str, optional
+            Side of the wind vector to start the ignition. Defaults to 'right'. Options are 'left' or 'right'.
 
-        Returns:
-            Pattern: Spatiotemporal ignition pattern
+        Returns
+        -------
+        Pattern
+            Spatiotemporal ignition pattern
         """
 
-        return self._generate_pattern(spacing=spacing, depth=depth, heat_depth=heat_depth)
+        return self._generate_pattern(spacing=spacing, depth=depth, heat_depth=heat_depth, side=side, heat_delay=heat_delay)
 
     def _init_paths(self, paths: dict, **kwargs) -> dict:
         """Initialize spatial part of the ignition paths.
@@ -58,9 +71,10 @@ class Strip(FiringBase):
         # `generate_pattern()` method of this class)
         depth = kwargs['depth']
         heat_depth = kwargs['heat_depth']
+        side = kwargs['side']
 
         # Extract the bounding extent of the firing area
-        bbox = self._burn_unit.get_bounds()
+        bbox = self._burn_unit.bounds
         x_min, y_min = bbox[:, 0].min(), bbox[:, 1].min()
         x_max, y_max = bbox[:, 0].max(), bbox[:, 1].max()
 
@@ -85,7 +99,7 @@ class Strip(FiringBase):
         # Initialize loop control parameters
         cur_heat = 0
         cur_igniter = 0
-        direction_toggle = True
+        direction_toggle = False if side == 'left' else True
 
         # For each start position, build a path and assign to a heat and igniter
         for i, x in enumerate(x_range):
@@ -101,12 +115,17 @@ class Strip(FiringBase):
 
             # Get lines or multipart lines in the same structure for looping below
             if isinstance(line, LineString):
-                line = [line]
+                line_list = [line]
             elif isinstance(line, MultiLineString):
-                line = list(line.geoms)
+                line_list = list(line.geoms)
+            # Edge case: In rare cases, the line along the top of the envelope becomes a point following
+            # the intersection (pretty sure this is a numerical precision issue). In this case, we need to
+            # just skip this path.
+            else:
+                continue
 
             # Assign the path to a heat, igniter and leg
-            for j, part in enumerate(line):
+            for j, part in enumerate(line_list):
                 paths['heat'].append(cur_heat)
                 paths['igniter'].append(cur_igniter)
                 paths['leg'].append(j)
