@@ -11,6 +11,8 @@ import warnings
 # Internal imports
 from .io import Projector, write_geojson, write_quicfire
 from .unit import BurnUnit
+from .errors import EPSGError
+from .personnel import IgnitionCrew
 
 # External imports
 import awkward as ak
@@ -18,33 +20,46 @@ import numpy as np
 import pandas as pd
 from shapely.errors import ShapelyDeprecationWarning
 from shapely import affinity
-from shapely.geometry import MultiPoint, MultiLineString, LineString
+from shapely.geometry import MultiPoint, MultiLineString, LineString, shape
 
 # Turn off Pandas copy warning (or figure out how to do it like the Panda wants)
 pd.options.mode.chained_assignment = None
 
-# Turn off the Shapely deprecation warning about about future removal
-# of the array interface. This happens when a Pandas takes a list of Shapely
-# geometries and casts to a list of ndarray objects. GeoPandas would fix this
-# but it's not worth the added complexity of GDAL C binary deps. This is fine
-# as long as we don't upgrade the Shapely req to v2.
+"""
+
+Turn off the Shapely deprecation warning about about future removal
+of the array interface. This happens when a Pandas takes a list of Shapely
+geometries and casts to a list of ndarray objects. GeoPandas would fix this
+but it's not worth the added complexity of GDAL C binary deps. This is fine
+as long as we don't upgrade the Shapely req to v2.
+
+"""
 warnings.filterwarnings("ignore", category=ShapelyDeprecationWarning)
 
 
 class Pattern:
+    """Patterns are objects that store the spatial and temporal components of a
+    firing technique.
 
-    def __init__(self, heat: list[int], igniter: list[int], leg: list[int],
-                 times: list[list[float]], geometry: list[LineString], epsg: int):
-        """Constructor
+    Parameters
+    ----------
+    heat : list[int]
+        Heat of the path
+    igniter : list[int]
+        Igniter id assigned to the path
+    leg : list[int])
+        Leg for the path
+    times : list[list[float]]
+        Coordinate arrival times for the path
+    geometry : list[LineString]
+        Path geometry
+    epsg : int
+        EPSG code for path geometry
+    """
 
-        Args:
-            heat (list[int]): Heat of the path
-            igniter (list[int]): Igniter id assigned to the path
-            leg (list[int]): Leg for the path
-            times (list[list[float]]): Coordinate arrival times for the path
-            geometry (list[LineString]): Path geometry
-            epsg (int): EPSG code for path geometry
-        """
+    def __init__(self, heat: list[int], igniter: list[int], leg: list[int], times: list[list[float]],
+                 geometry: list[LineString], epsg: int,
+                 ):
 
         self.heat = heat
         self.igniter = igniter
@@ -64,58 +79,66 @@ class Pattern:
         """Alternative constructor for initializing a Pattern object with a dictionary
         of path parameters
 
-        Args:
-            paths_dict (dict): Dictionary of path parameters
-            burn_unit (BurnUnit): Burn unit object associated with the ignition pattern
+        Parameters
+        ----------
+        paths_dict : dict
+            Dictionary of path parameters
+        epsg : Int
+            EPSG code of path geometries
 
-        Returns:
-            Pattern: A new instance of Pattern
+        Returns
+        -------
+        Pattern
+            A new instance of Pattern
         """
 
+        paths_dict["geometry"] = [shape(x) for x in paths_dict["geometry"]]
         return cls(
-            paths_dict['heat'],
-            paths_dict['igniter'],
-            paths_dict['leg'],
-            paths_dict['times'],
-            paths_dict['geometry'],
-            epsg
+            paths_dict["heat"],
+            paths_dict["igniter"],
+            paths_dict["leg"],
+            paths_dict["times"],
+            paths_dict["geometry"],
+            epsg,
         )
 
     def to_dict(self) -> dict:
         """Returns the Pattern path parameters as a dictionary
 
-        Returns:
-            dict: Pattern path parameters
+        Returns
+        -------
+        dict
+            Pattern path parameters
         """
 
         return {
-            'heat': self.heat,
-            'igniter': self.igniter,
-            'leg': self.leg,
-            'times': self.times,
-            'geometry': self.geometry
+            "heat": self.heat,
+            "igniter": self.igniter,
+            "leg": self.leg,
+            "times": self.times,
+            # convert to geoJSON for storage
+            "geometry": [x.__geo_interface__ for x in self.geometry],
         }
 
     @staticmethod
     def empty_path_dict() -> dict:
         """Helper method for initializing a path parameter dictionary
 
-        Returns:
-            dict: Empty path dictionary
+        Returns
+        -------
+        dict
+            Empty path dictionary
         """
 
-        return {
-            'heat': [],
-            'igniter': [],
-            'leg': [],
-            'geometry': []
-        }
+        return {"heat": [], "igniter": [], "leg": [], "geometry": []}
 
     def to_json(self) -> dict:
         """Write the Pattern to a GeoJSON dictionary
 
-        Returns:
-            dict: Timestamped GeoJSON representation of the firing pattern
+        Returns
+        -------
+        dict
+            Timestamped GeoJSON representation of the firing pattern
         """
 
         # Copy the times array
@@ -136,25 +159,38 @@ class Pattern:
         times = (times * 1000) + (unix_time() * 1000)
 
         # Set the props and styling
-        props = {'heat': self.heat, 'igniter': self.igniter,
-                 'leg': self.leg, 'times': times.to_list()}
-        style = {'icon': 'circle', 'style': {
-            'color': '#ff0000', 'radius': 1}}
+        props = {
+            "heat": self.heat,
+            "igniter": self.igniter,
+            "leg": self.leg,
+            "times": times.to_list(),
+        }
+        style = {"icon": "circle", "style": {"color": "#ff0000", "radius": 1}}
 
         # Send off to the GeoJSON writer and return
-        return write_geojson(self.geometry, self.epsg, properties=props, style=style,
-                             elapsed_time=self.elapsed_time)
+        return write_geojson(
+            self.geometry,
+            self.epsg,
+            properties=props,
+            style=style,
+            elapsed_time=self.elapsed_time,
+        )
 
     def translate(self, x_off: float, y_off: float) -> Pattern:
         """Translate pattern geometry along the x and y axis by the supplied
         x and y offset amounts.
 
-        Args:
-            x_off (float): Offset along the x axis
-            y_off (float): Offset along the y axis
+        Parameters
+        ----------
+        x_off : float
+            Offset along the x axis
+        y_off : float
+            Offset along the y axis
 
-        Returns:
-            Pattern: New Pattern object with translated geometries
+        Returns
+        -------
+        Pattern
+            New Pattern object with translated geometries
         """
 
         # Translate the path geometries
@@ -171,19 +207,27 @@ class Pattern:
         # Return the new Pattern object
         return obj_copy
 
-    def to_quicfire(self, burn_unit: BurnUnit, filename: str = None, time_offset=0, resolution: int = 1, dst_epsg: int = None) -> None | str:
+    def to_quicfire(self, burn_unit: BurnUnit, filename: str = None, time_offset=0,
+                    resolution: int = 1, dst_epsg: int = None) -> None | str:
         """Write paths dictionary to QUIC-fire ignition file format.
 
-        Args:
-            burn_unit (BurnUnit): Burn unit that defines the extent of the QF simulation
-            filename (str, optional): If provided, write the ignition file to the
-                filename. Defaults to None.
-            time_offset (float, optional): Time offset to add to the ignition times. Defaults to 0.
-            resolution (int, optional): Horizontal resolution of QUIC-fire domain (meters). Defaults to 1.
-            dst_epsg (int, optional): EPSG code for the destination projection. Defaults to None.
+        Parameters
+        ----------
+        burn_unit : BurnUnit
+            Burn unit that defines the extent of the QF simulation
+        filename : str, optional
+            If provided, write the ignition file to the filename. Defaults to None.
+        time_offset : float, optional
+            Time offset to add to the ignition times. Defaults to 0.
+        resolution : int, optional
+            Horizontal resolution of QUIC-fire domain (meters). Defaults to 1.
+        dst_epsg : int, optional
+            EPSG code for the destination projection. Defaults to None.
 
-        Returns:
-            None | str: None if filename provided, string containing the ignition file if not.
+        Returns
+        -------
+        None | str
+            None if filename provided, string containing the ignition file if not.
         """
 
         times = self.times.copy()
@@ -208,7 +252,7 @@ class Pattern:
             geometry = reproj_geoms
 
         # Translate pattern geometry to the origin or the CRS according to the burn unit extent
-        lower_left = domain.get_bounds().min(axis=0)
+        lower_left = domain.bounds.min(axis=0)
         trans_geoms = []
         for geom in geometry:
             trans_geoms.append(affinity.translate(
@@ -217,12 +261,75 @@ class Pattern:
 
         # Check if filename was provided and write to it if so
         if filename:
-            with open(filename, 'w') as f:
-                f.write(write_quicfire(geometry, times,
-                        self.elapsed_time, resolution=resolution))
+            with open(filename, "w") as f:
+                f.write(
+                    write_quicfire(
+                        geometry, times, self.elapsed_time, resolution=resolution
+                    )
+                )
         # Otherwise return QF ignition file to client as string
         else:
-            return write_quicfire(geometry, times, self.elapsed_time, resolution=resolution)
+            return write_quicfire(
+                geometry, times, self.elapsed_time, resolution=resolution
+            )
+
+    def merge(self, pattern: Pattern, time_offset: float = 0, inplace: bool = False) -> Pattern:
+        """Merge an input pattern with self
+
+        Parameters
+        ----------
+        pattern : Pattern
+            Input pattern to be merged into self
+        time_offset : float
+            Time offset between patterns (seconds)
+        inplace : bool, optional
+            Overwrites Pattern object if true, otherwise return new Pattern object. Defaults to True.
+
+        Raises
+        ------
+        EPSGError.non_equivalent: EPSG code for input pattern does not match self.epsg
+
+        Returns
+        -------
+        Pattern
+            Merged pattern object
+        """
+
+        # Check that the EPSG codes are the same
+        if pattern.epsg != self.epsg:
+            raise EPSGError.non_equivalent
+
+        # Get an empty path dictionary
+        merged_dict = self.empty_path_dict()
+
+        # Delay times in the second pattern by the elapsed time of the first pattern and offset
+        delayed_times = (
+            ak.max(self.times) + time_offset + ak.Array(pattern.times)
+        ).to_list()
+
+        # Build merged pattern attributes
+        merged_dict["heat"] = self.heat + pattern.heat
+        merged_dict["igniter"] = self.igniter + pattern.igniter
+        merged_dict["leg"] = self.leg + pattern.leg
+        merged_dict["times"] = self.times + delayed_times
+        merged_dict["geometry"] = self.geometry + pattern.geometry
+
+        # Instantiate a new Pattern object
+        merged_pattern = self.from_dict(merged_dict, self.epsg)
+
+        # Compute summed elapse time
+        merged_pattern.elapsed_time = (
+            self.elapsed_time + time_offset + pattern.elapsed_time
+        )
+
+        # If inplace then overwrite self with the merged pattern
+        if inplace:
+            self = merged_pattern
+
+        # If inplace is False then return the merged pattern. This is outside
+        # of the if statement so that the merged pattern is still return in case
+        # the user sets the method to a variable
+        return merged_pattern
 
 
 class TemporalPropagator:
@@ -231,7 +338,9 @@ class TemporalPropagator:
     through their coordinates.
     """
 
-    def __init__(self, spacing: float = 0, sync_end_time: bool = False, return_trip: bool = False):
+    def __init__(
+        self, spacing: float = 0, sync_end_time: bool = False, return_trip: bool = False
+    ):
         """Class constructor
 
         Args:
@@ -247,24 +356,33 @@ class TemporalPropagator:
         self.sync_end_time = sync_end_time
         self.return_trip = return_trip
 
-    def forward(self, paths, ignition_crew):
+    def forward(self, paths: dict, ignition_crew: IgnitionCrew, heat_delay: float) -> None:
+        """Compute and store time values for the provided paths dicitionary and its respective
+           ignition crew.
 
+        Args:
+            paths (dict): Dictionary containing the paths generated by a given technique
+            ignition_crew (IgnitionCrew): The set of ignitors responsible to generating the paths
+            heat_delay (float, optional): Time delay between ignition heats.
+        Returns:
+            None: Inplace operation.
+        """
         # Create a Pandas DataFrame from the initialized paths dictionary
         self.paths = pd.DataFrame(paths)
         self.ignition_crew = ignition_crew
 
-        # TODO: #6 Check for geometry, heat, igniter and leg columns in Propagator.
         # Geometry must of type LineString
 
         # Setup some new dataframe columns
-        self.paths['start_time'] = 0
-        self.paths['end_time'] = 0
-        self.paths['times'] = None
-        self.paths['times'] = self.paths['times'].astype('object')
+        self.paths["start_time"] = 0
+        self.paths["end_time"] = 0
+        self.paths["times"] = None
+        self.paths["times"] = self.paths["times"].astype("object")
 
         # Sort dataframe by heat, igniter, leg (in that order)
-        self.paths.sort_values(by=['heat', 'igniter', 'leg'], ascending=[
-            True, True, True], inplace=True)
+        self.paths.sort_values(
+            by=["heat", "igniter", "leg"], ascending=[True, True, True], inplace=True
+        )
 
         # Run the initial forward pass through the paths
         self._init_path_time(self.spacing)
@@ -273,17 +391,20 @@ class TemporalPropagator:
         if self.sync_end_time:
             self._sync_end_time()
 
+        # Offset time between heats
+        if heat_delay:
+            self._delay_heats(heat_delay)
+
         # Compute the arrival time for each coordinate in each path
         self._compute_arrival_times()
 
         # Drop the intermidary columns
-        self.paths.drop(['start_time', 'end_time'],
-                        axis=1, inplace=True)
+        self.paths.drop(["start_time", "end_time"], axis=1, inplace=True)
 
-        return self.paths.to_dict(orient='list')
+        return self.paths.to_dict(orient="list")
 
     def _init_path_time(self, spacing: float):
-        """ Helper method to run the initial time propagation.
+        """Helper method to run the initial time propagation.
 
         Args:
             spacing (float): Stagger spacing between igniters in a heat
@@ -303,8 +424,11 @@ class TemporalPropagator:
             if k != 0:
 
                 # Get the previous path leg
-                prev_leg = self.paths.loc[(self.paths.heat == i) & (
-                    self.paths.igniter == j) & (self.paths.leg == k-1)]
+                prev_leg = self.paths.loc[
+                    (self.paths.heat == i)
+                    & (self.paths.igniter == j)
+                    & (self.paths.leg == k - 1)
+                ]
 
                 # Grab the geometries of the previous and current leg
                 prev_leg_geom = prev_leg.geometry.iloc[0]
@@ -325,8 +449,9 @@ class TemporalPropagator:
             elif (i != 0) and (j == 0):
 
                 # Get the maximum end time from all igniters in the previous heat
-                prev_heat_max_end_time = self.paths.loc[self.paths.heat ==
-                                                        i-1, 'end_time'].max()
+                prev_heat_max_end_time = self.paths.loc[
+                    self.paths.heat == i - 1, "end_time"
+                ].max()
 
                 # The current igniter's start time is the previous heat max end time
                 path.start_time = prev_heat_max_end_time
@@ -348,10 +473,12 @@ class TemporalPropagator:
                 # Get the previous and current igniters (again, no need to use
                 # k the leg index, these will all be leg 0 due to the catch
                 # above)
-                cur_igniter = self.paths.loc[(
-                    self.paths.heat == i) & (self.paths.igniter == j)]
-                prev_igniter = self.paths.loc[(
-                    self.paths.heat == i) & (self.paths.igniter == j-1)]
+                cur_igniter = self.paths.loc[
+                    (self.paths.heat == i) & (self.paths.igniter == j)
+                ]
+                prev_igniter = self.paths.loc[
+                    (self.paths.heat == i) & (self.paths.igniter == j - 1)
+                ]
 
                 # Get the previous igniter's start time
                 prev_start_time = prev_igniter.start_time.values[0]
@@ -359,7 +486,8 @@ class TemporalPropagator:
                 # Compute the offset due to stagger spacing and incongruence
                 # along a coordinate frame axis
                 path.start_time = prev_start_time + self._get_offset(
-                    prev_igniter, cur_igniter, spacing, velocity)
+                    prev_igniter, cur_igniter, spacing, velocity
+                )
 
             # The last condition we have caught yet is the first igniter of the
             # first heat. That igniter gets a start time of zero
@@ -376,12 +504,18 @@ class TemporalPropagator:
         # Depending of the arrange of igniter start positions it is
         # possible that the minimum start time is less than zero. We fix
         # that by adding the minimum back to all start and end times
-        min_start_time = self.paths['start_time'].min()
+        min_start_time = self.paths["start_time"].min()
         if min_start_time != 0:
-            self.paths['start_time'] -= min_start_time
-            self.paths['end_time'] -= min_start_time
+            self.paths["start_time"] -= min_start_time
+            self.paths["end_time"] -= min_start_time
 
-    def _get_offset(self, prev_igniter: pd.Series, cur_igniter: pd.Series, spacing: float, velocity: float) -> float:
+    def _get_offset(
+        self,
+        prev_igniter: pd.Series,
+        cur_igniter: pd.Series,
+        spacing: float,
+        velocity: float,
+    ) -> float:
         """Compute the offset time between the previous and current igniters
 
         Args:
@@ -414,16 +548,14 @@ class TemporalPropagator:
 
         # Now we can project the a vector onto the unit b vector and
         # compute its magnitude which give the offset distance
-        offset_distance = a_vec.dot(b_vec/np.linalg.norm(b_vec))
+        offset_distance = a_vec.dot(b_vec / np.linalg.norm(b_vec))
 
         # Add the stagger spacing distance and divide by velocity
         # for travel time
         return (spacing + offset_distance) / velocity
 
     def _sync_end_time(self):
-        """ Helper method to synchronize end times"""
-
-        # TODO: #7 What about multileg ignition paths when synchronizing end times?
+        """Helper method to synchronize end times"""
 
         # Get the unique heat indecies
         heats = np.sort(self.paths.heat.unique())
@@ -445,6 +577,34 @@ class TemporalPropagator:
             # Place the current heat back in the dataframe
             self.paths[self.paths.heat == i] = cur_heat
 
+    def _delay_heats(self, heat_delay: float) -> None:
+        """Offset the time between respective heats
+
+        Args:
+            heat_delay (float): Time offset between heats
+        """
+
+        # Get the unique heat indecies
+        heats = np.sort(self.paths.heat.unique())
+
+        # Loop over heats and accumulate the heat delay then apply to the start and end times
+        # of each igniter in the heat
+        for i in heats:
+            # We don't want to delay the first heat
+            if i > 0:
+                # Grab the dataframe rows for the current heat
+                cur_heat = self.paths[self.paths.heat == i]
+
+                # Accumulate time by multiplying the heat index by the heat delay
+                time_diff = i*heat_delay
+
+                # Loop over igniters in the heat and apply the time difference
+                for idx, _ in cur_heat.iterrows():
+
+                    # Adjust the start and time times
+                    self.paths.loc[idx, "start_time"] += time_diff
+                    self.paths.loc[idx, "end_time"] += time_diff
+
     def _compute_arrival_times(self):
         """For each path, ask the igniter what line type they are
         suppose to lay down and call the respective methods.
@@ -452,14 +612,14 @@ class TemporalPropagator:
 
         for index, path in self.paths.iterrows():
             cur_igniter = self.ignition_crew[path.igniter]
-            if cur_igniter.interval == 0:
+            if (cur_igniter.gap_length == None) and (cur_igniter.dash_length == None):
                 self._lines(index, path, cur_igniter.velocity)
-            elif cur_igniter.interval < 0:
+            elif cur_igniter.dash_length != None:
                 self._dashes(index, path, cur_igniter.velocity,
-                             -cur_igniter.interval)
+                             cur_igniter.gap_length, cur_igniter.dash_length)
             else:
                 self._dots(index, path, cur_igniter.velocity,
-                           cur_igniter.interval)
+                           cur_igniter.gap_length)
 
     def _lines(self, index: int, path: pd.Series, velocity: float):
         """Compute arrival times along each igniter's coordinate sequence
@@ -480,17 +640,17 @@ class TemporalPropagator:
 
             # Get the next coordinate in the sequence and compute the distance
             # between it and the current coordinate
-            next_xy = coords[i+1]
+            next_xy = coords[i + 1]
             meters = np.linalg.norm(xy - next_xy)
 
             # Update the arrival time and add to the times array for the
             # current path
-            arrival_time = arrival_time + (meters/velocity)
+            arrival_time = arrival_time + (meters / velocity)
             path_times.append(arrival_time)
 
-        self.paths.at[index, 'times'] = path_times
+        self.paths.at[index, "times"] = path_times
 
-    def _dashes(self, index: int, path: pd.Series, velocity: float, interval: float):
+    def _dashes(self, index: int, path: pd.Series, velocity: float, gap_length: float, dash_length: float):
         """Compute arrival times along each igniter's coordinate sequence
         by picking up and putting down ignitions (dashes)
         """
@@ -501,11 +661,24 @@ class TemporalPropagator:
 
         # Get a range of distances along the length of the path spaced by
         # the ignition rate
-        distances = np.arange(0, path.geometry.length, interval)
+        if not gap_length:
+            distances = np.arange(0, path.geometry.length, dash_length)
+        else:
+            distances = []
+            sum_length = 0
+            toggle = True
+            while sum_length < path.geometry.length:
+                distances.append(sum_length)
+                if toggle:
+                    sum_length += dash_length
+                else:
+                    sum_length += gap_length
+                toggle ^= True
 
         # Interpolate points at those distance along the path
-        points = MultiPoint([path.geometry.interpolate(distance)
-                             for distance in distances])
+        points = MultiPoint(
+            [path.geometry.interpolate(distance) for distance in distances]
+        )
 
         # Set the arrays for the current path arrival times and
         # dash geometries
@@ -521,11 +694,11 @@ class TemporalPropagator:
 
             # Get the current and next point coordinates
             xy = np.array(point.coords)[0]
-            next_xy = np.array(point_geoms[i+1].coords)[0]
+            next_xy = np.array(point_geoms[i + 1].coords)[0]
 
             # Comput the distance between points and the travel time
             meters = np.linalg.norm(xy - next_xy)
-            end_time = float(start_time + (meters/velocity))
+            end_time = float(start_time + (meters / velocity))
 
             # IF fire is active for this segment then add start and end
             # times to the current path arrival time arrray and append
@@ -542,10 +715,10 @@ class TemporalPropagator:
         # Add the current path arrival times to the global times array
         # and create a multileg line string and append to the new geometry
         # array
-        self.paths.at[index, 'times'] = path_times
-        self.paths.at[index, 'geometry'] = MultiLineString(line_segs)
+        self.paths.at[index, "times"] = path_times
+        self.paths.at[index, "geometry"] = MultiLineString(line_segs)
 
-    def _dots(self, index: int, path: pd.Series, velocity: float, interval: float):
+    def _dots(self, index: int, path: pd.Series, velocity: float, gap_length: float):
         """Compute arrival times along each igniter's coordinate sequence
         for interpolated equally spaced points (dots)
         """
@@ -554,11 +727,12 @@ class TemporalPropagator:
 
         # Get a range of distances along the length of the path space by
         # the ignition rate
-        distances = np.arange(0, path.geometry.length, interval)
+        distances = np.arange(0, path.geometry.length, gap_length)
 
         # Interpolate points at those distance along the path
-        points = MultiPoint([path.geometry.interpolate(distance)
-                             for distance in distances])
+        points = MultiPoint(
+            [path.geometry.interpolate(distance) for distance in distances]
+        )
 
         # Set the initial arrival time
         path_times = [arrival_time]
@@ -568,16 +742,16 @@ class TemporalPropagator:
 
             # Get the current and next points
             xy = np.array(point.coords)
-            next_xy = np.array(points[i+1].coords)
+            next_xy = np.array(points[i + 1].coords)
 
             # Compute the distance between points and travel time
             meters = np.linalg.norm(xy - next_xy)
-            arrival_time = arrival_time + (meters/velocity)
+            arrival_time = arrival_time + (meters / velocity)
 
             # Add the arrival time to the current path's time array
             path_times.append(arrival_time)
 
         # Add the current path's time array to the global time array and
         # add the multipar point geometry to the new geometries array
-        self.paths.at[index, 'times'] = path_times
-        self.paths.at[index, 'geometry'] = points
+        self.paths.at[index, "times"] = path_times
+        self.paths.at[index, "geometry"] = points
