@@ -6,3 +6,7 @@ class CrewSizeWarning(UserWarning):
     only_using_two = f'{base_msg} 2. Only the first two igniters in the crew will be used'
     cloning_first = f'{base_msg} 2. The first igniter was dulicated to increase ' \
         'the crew to the minimum required size.'
+
+
+class IgniterWarning(UserWarning):
+    velocity_warning = 'Igniter velocity is above reasonable rates'
\ No newline at end of file
